# generated by datamodel-codegen:
#   filename:  openapi.json

from __future__ import annotations

from enum import Enum
from typing import List, Literal, Optional, Union

from pydantic import BaseModel, Extra, Field


class LoincCodeFilter(BaseModel):
    class Config:
        extra = Extra.forbid

    loincCodes: List[str]


class TrainingApproachBase(BaseModel):
    class Config:
        extra = Extra.forbid

    id: str
    """
    UUID uniquely identifying this training approach.
    """
    name: str
    description: str


class CategoricalParameterSpace(BaseModel):
    class Config:
        extra = Extra.forbid

    type: Literal["categorical"]
    name: str
    values: List[str]


class NumericScale(Enum):
    auto = "auto"
    linear = "linear"
    log = "log"
    reverseLog = "reverseLog"


class NumericParameterSpace(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str
    min: float
    max: float
    scale: NumericScale


class ContinuousParameterSpace(NumericParameterSpace):
    type: Literal["continuous"]


class IntegerParameterSpace(NumericParameterSpace):
    type: Literal["integer"]


class ParameterSpace(BaseModel):
    __root__: Union[
        CategoricalParameterSpace, ContinuousParameterSpace, IntegerParameterSpace
    ]


class OptimizationObjective(BaseModel):
    class Config:
        extra = Extra.forbid

    direction: Literal["minimize", "maximize"]
    metric: str


class MetricDefinition(BaseModel):
    """
    Camel-cased mirror of https://docs.aws.amazon.com/sagemaker/latest/APIReference/API_MetricDefinition.html.
    """

    class Config:
        extra = Extra.forbid

    name: str
    regex: str


class TuningJobTrainingApproach(TrainingApproachBase):
    type: Literal["tuningJob"]
    trainingImage: str
    metricDefinitions: List[MetricDefinition]
    objective: OptimizationObjective
    """
    The metric to optimize during the hyperparameter tuning process.
    """
    searchSpace: List[ParameterSpace]
    """
    The hyperparameter search space to explore during hyperparameter tuning.
    """
    maxTrials: Optional[int] = Field(None, ge=1, le=30)
    """
    The maximum number of candidate models to consider in a given model run.
    """


class TrainingApproach(BaseModel):
    __root__: TuningJobTrainingApproach


class DeployApproachBase(BaseModel):
    class Config:
        extra = Extra.forbid

    inferenceImage: str


class EdgeDeployApproach(DeployApproachBase):
    type: Literal["edge"]


class DeployApproach(BaseModel):
    __root__: EdgeDeployApproach


class Split(BaseModel):
    class Config:
        extra = Extra.forbid

    n: float
    """
    The number of examples in this split.
    """
    start: float
    """
    Timestamp marking the beginning of the data for this split (inclusive). Expressed as milliseconds since the UTC epoch.
    """
    end: float
    """
    Timestamp marking the end of the data for this split (exclusive). Expressed as milliseconds since the UTC epoch.
    """
    uri: str = Field(..., regex="^s3:\\/\\/.+$")
    """
    S3 URI of where a copy of this exact split is saved.
    """


class Metric(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str
    """
    The name of the metric e.g. "Cross Entopy Loss", "Accuracy", "F1 Macro", etc.
    """
    value: float
    stage: Literal["training", "evaluation"]
    """
    The stage of the model run this metric was computed in. If `training`, the metric could have been computed over the train or val set. If `evaluation`, the metric was computed over the test set.
    """


class ApprovalChoice(Enum):
    approved = "approved"
    rejected = "rejected"


class ApprovalDecisionBase(BaseModel):
    class Config:
        extra = Extra.forbid

    timestamp: float
    """
    Timestamp of when the decision was made. Expressed as milliseconds since the UTC epoch.
    """
    decision: ApprovalChoice


class SystemApprovalDecision(ApprovalDecisionBase):
    actor: Literal["system"]


class UserApprovalDecision(ApprovalDecisionBase):
    actor: Literal["user"]
    user: str


class ApprovalDecision(BaseModel):
    __root__: Union[SystemApprovalDecision, UserApprovalDecision]


class RunSplits(BaseModel):
    """
    Information about the different splits of the dataset used to train this model version.
    """

    class Config:
        extra = Extra.forbid

    train: Split
    val: Split
    test: Split


class RunMetrics(BaseModel):
    class Config:
        extra = Extra.forbid

    challenger: List[Metric]
    """
    Metrics about how the model version trained in this run (the challenger) performed on this run's test set.
    """
    champion: List[Metric]
    """
    Metrics about how the current best model (the champion) performed on this run's test set.
    """


class Parameter(BaseModel):
    """
    A hyperparameter value. The values themselves are represented as strings.
    """

    class Config:
        extra = Extra.forbid

    name: str
    value: str


class ImageSegmentationProblem(BaseModel):
    class Config:
        extra = Extra.forbid

    problemType: Literal["imgSeg"]
    imageFilter: LoincCodeFilter
    """
    Used to filter through patient data to identify the input images for the model.
    """
    maskFilter: LoincCodeFilter
    """
    Used to filter through patient data to identify the image segmentation mask training label images for the model.
    """


class ImageClassificationProblem(BaseModel):
    class Config:
        extra = Extra.forbid

    problemType: Literal["imgClf"]
    imageFilter: LoincCodeFilter
    """
    Used to filter through patient data to identify the input images for the model.
    """


class MlProblemDefinition(BaseModel):
    __root__: Union[ImageSegmentationProblem, ImageClassificationProblem]


class ModelConfigBase(BaseModel):
    class Config:
        extra = Extra.forbid

    name: str
    description: str
    problemDefinition: MlProblemDefinition


class ModelConfigInput(ModelConfigBase):
    trainingApproach: Optional[TrainingApproach] = None
    """
    A custom training approach to use. If not supplied, a default will be used appropriate to the problem type.
    """
    deployApproach: Optional[DeployApproach] = None
    """
    A custom deploy approach to use. If not supplied, a default will be used appropriate to the problem type and deploy type.
    """


class ModelConfig(ModelConfigBase):
    id: str
    """
    UUID uniquely identifying this model config.
    """
    accountId: str
    trainingApproach: TrainingApproach
    deployApproach: DeployApproach
    deployedId: Optional[str] = None
    """
    The ID of the model version currently deployed for this model config.
    """


class ModelRun(BaseModel):
    class Config:
        extra = Extra.forbid

    id: str
    """
    UUID uniquely identifying this model run.
    """
    slug: str
    """
    Human-readable slug that acts as an auto-generated name for the run. This field is not guaranteed to be unique, so cannot be used reliably as a unique identifier.
    """
    modelId: str
    """
    ID of the model config this run was produced for.
    """
    accountId: str
    modelArtifactUri: Optional[str] = None
    """
    S3 URI location where the final trained model artifact produced by this run is saved to.
    """
    status: Literal["running", "succeeded", "failed"]
    error: Optional[str] = None
    """
    If `status` is 'failed', gives the reason why the run failed.
    """
    start: float
    """
    Timestamp of when the run started. Expressed as milliseconds since the UTC epoch.
    """
    end: Optional[float] = None
    """
    Timestamp of when the run ended. Expressed as milliseconds since the UTC epoch.
    """
    splits: Optional[RunSplits] = None
    hyperparameters: List[Parameter]
    """
    The hyperparameters used to train the model version created by this run.
    """
    metrics: RunMetrics
    approvals: List[ApprovalDecision]
    """
    Decisions made by various actors representing whether they think this model version should be used in production and become the new champion.
    """
    problemDefinition: MlProblemDefinition
    trainingApproach: TrainingApproach
